definition module combinators

($) infixr 0
($) f g :== f g

import StdEnv, parse

wantModule` :: !Bool !Ident !Position !Bool !*HashTable !*File !SearchPaths (ModTimeFunction *Files) !*Files (Simple_0_1 ParsedModule (ParsedModule,*State))
	-> (!Bool, !ParsedModule, !*HashTable, !*File, !*Files)

:: *State

:: Simple_parser *a *b	:== (*State->a)				-> *State->b
:: Try_parser *a *b		:== (*State->a) (*State->b)	-> *State->b

:: Simple_0_0		*e	:== Simple_parser	e				e
:: Simple_0_1 a		*e	:== Simple_parser	(a->e)			e
:: Simple_0_2 a	b	*e	:== Simple_parser	(b->*(a->e))	e
:: Simple_1_1 x a	*e	:== Simple_parser	(a->e)			(x->e)
:: Simple_2_1 x y a	*e	:== Simple_parser	(a->e)			(y->*(x->e))

:: Try_0_0			*e	:== Try_parser	e		e
:: Try_0_1	 x		*e	:== Try_parser 	(x->e)	e
:: Try_1_1	 y x	*e	:== Try_parser 	(x->e)	(y->e)


// state modification functions
normalContext		:: Simple_0_0 e
typeContext			:: Simple_0_0 e
setLayoutTrue		:: Simple_0_0 e
underscoreAllowed	:: !(Simple_0_1 a e) -> Simple_0_1 a e
into_hashTable		:: !IdentClass -> Simple_1_1 String Ident e

// simple parsers
basicToken			:: Try_0_1 BasicValue e
constructorToken	:: Try_0_1 String e		// like Char Id /\ 
variableToken		:: Try_0_1 String e		// like id var0 _ _id
codeLine			:: Try_0_1 String e		// anything but }

class Keyword a
where
	keyword			:: !a -> Try_0_0 e
	checkKeyword	:: !a -> Try_0_0 e
	expected		:: !a -> Simple_0_0 e	//error handling

instance Keyword String
instance Keyword Char

checkDefinitionEnd	:: Try_0_0 e		// ; } \0
checkFileEnd		:: Try_0_0 e		// \0
doubleColonToken 	:: Try_0_0 e

epsilon continue			:== continue		//:: Simple_0_0
succeed success fail		:== success			//:: Try_0_0


// combinators
apply_0 :: !(a) -> Simple_parser (a->*e) e
apply_1 :: !(x->a) -> Simple_parser (a->*e) (x->*e)
apply_2 :: !(y x->a) -> Simple_parser (a->*e) (x->*(y->*e))
apply_3 :: !(z y x->a) -> Simple_parser (a->*e) (x->*(y->*(z->*e)))
apply_4 :: !(v z y x->a) -> Simple_parser (a->*e) (x->*(y->*(z->*(v->*e))))
apply_5 :: !(w v z y x->a) -> Simple_parser (a->*e) (x->*(y->*(z->*(v->*(w->*e)))))
apply_6 :: !(l w v z y x->a) -> Simple_parser (a->*e) (x->*(y->*(z->*(v->*(w->*(l->*e))))))

drop_1 continue state _		:== continue state		//:: ContN -> Simple_parser(N-1)

(|>)	infixr 2	//:: TryN GiveN -> GiveN
(|>)	f g			:== \success= f success (g success)

(<|>)	infixl 2	//:: TryNM TryNM -> TryNM
(<|>)	f g			:== \success fail= f success (g success fail)

(&>)	infixr 4	//:: TGN TGM -> TG(N+M)
(&>)	f g			:== \success= f (g success)

(<&>)	infixr 4	//:: Try_0_0 TryN -> TryN
(<&>)	f g			:== \success fail= f (g success fail) fail

(<&+>)	infixr 4	//:: Try_0_1 TryN -> Try_parser(N+1)
(<&+>)	f g			:== \success fail= f (g success (drop_1 fail)) fail

toSafe				:: !(Try_parser a b) -> Try_parser a b

zeroOrMore			:: !(Try_0_1 a e) -> Simple_0_1 [a] e
listWith			:: !(Simple_0_1 a e) !(Try_0_0 e) !(Try_0_0 e) -> Simple_0_1 [a] e
groupWith			:: !(Simple_0_1 a e) !(Try_0_0 e) -> Simple_0_1 [a] e


// error handling
error				:: !String -> Simple_0_0 e
warning				:: !String -> Simple_0_0 e

getFilePosition		:: Simple_0_1 FilePosition e
getPosition`		:: Simple_0_1 Position e
getLineAndColumn	:: Simple_0_1 LineAndColumn e
getFileName			:: Simple_0_1 String e

