implementation module combinators

import StdEnv, StdLib, RWSDebug
import hashtable

wantModule` :: !Bool !Ident !Position !Bool !*HashTable !*File !SearchPaths (ModTimeFunction *Files) !*Files (Simple_0_1 ParsedModule (ParsedModule,*State))
	-> (!Bool, !ParsedModule, !*HashTable, !*File, !*Files)
wantModule` iclmodule file_id import_file_position support_generics hash_table error searchPaths modtimefunction files parse
	# file_name= file_id.id_name +++ if iclmodule ".icl" ".dcl"

	= case fopenInSearchPaths file_name searchPaths FReadData modtimefunction files of
		(No, files)
			= let mod = { mod_name = file_id,  mod_modification_time = "", mod_type = MK_None, mod_imports = [], mod_imported_objects = [], mod_defs = [] } in
			  (False, mod, hash_table, error <<< "Error " <<< import_file_position <<< ": "  <<< file_name <<< " could not be imported\n", files)
		(Yes (input_file, modification_time), files)
			#! hash_table= set_hte_mark (if iclmodule 1 0) hash_table

			#! state= (nextLine &> skipSpace) id
					{ tabsize			= 4
					, line				= ""
					, row				= 0
					, column			= 0
					, offside_column	= -10
			
					, error_file		= error
					, okey				= True
					, skipping			= False
			
					, underscoreAllowed = file_name.[0]=='_'
					, input				= input_file
					, input_name		= file_name
			
					, hashTable			= hash_table
					, context			= NormalContext
					}

			#! (mod,{input,error_file,okey,hashTable})= parse (\state mod=(mod,state)) state
			#! mod= {mod& mod_modification_time= modification_time}
			#! (_,files)= fclose input files
			#! hashTable= set_hte_mark 0 hashTable
			= (okey, mod, hashTable, error_file, files)

 
///////////////////

:: *State =	{ input				:: !*File
			, input_name 		:: !String
			, line				:: !String
			, row				:: !Int
			, column			:: !Int
			, offside_column	:: !Int

			, error_file		:: !*File
			, skipping			:: !Bool
			, okey				:: !Bool

			, tabsize			:: !Int
			, underscoreAllowed :: !Bool
			, context			:: !Int
			, hashTable			:: !*HashTable
			}

/////////////////////

//modifyStateWith :: (*State->*State) -> Simple_0_0 e
modifyStateWith f continue state :== continue (f state)

writeMessage f state	= {state&error_file= f state.error_file}

normalContext :: Simple_0_0 e
normalContext= modifyStateWith \state= {state& context=NormalContext}

typeContext :: Simple_0_0 e
typeContext= modifyStateWith \state= {state& context=TypeContext`}

setLayoutTrue :: Simple_0_0 e
setLayoutTrue= modifyStateWith \state= {state&offside_column= -1}

underscoreAllowed	:: !(Simple_0_1 a e) -> Simple_0_1 a e
underscoreAllowed f =
	\continue state=:{underscoreAllowed}= f (\state= continue {state& underscoreAllowed=underscoreAllowed}) {state& underscoreAllowed=True}

into_hashTable :: !IdentClass -> Simple_1_1 String Ident e
into_hashTable ident_class = f where
	f continue state=:{hashTable} name
		#! ({boxed_ident=ident}, hashTable) = putIdentInHashTable name ident_class hashTable
		= continue {state & hashTable = hashTable} ident

clearError state	= {state& skipping=False}
setError state		= {state& okey=False, skipping=True}

checkError :: Try_0_0 e
checkError= \success fail state=:{skipping} = if skipping success fail state

///////
setOffside prev state= renewSemicolon {state&offside_column=prev}

beginGroupToken continue state=:{offside_column= -10} = continue -10 state
beginGroupToken continue state=:{column,offside_column} = continue offside_column (dropSemicolon {state&offside_column=column})

dropSemicolon state = {state& offside_column= f state.offside_column} where f i= if (i> -10) (-20-i) i
renewSemicolon state = {state& offside_column= f state.offside_column} where f i= if (i< -10) (-20-i) i

checkColumnWith f :== g where g success fail state=:{column,offside_column} = if (f column offside_column) success fail state

//////
skipChar n :== modifyStateWith \state= {state&column=state.column+n}

getFilePosition :: Simple_0_1 FilePosition e
getFilePosition= \continue state=:{column,row} = continue state {fp_line=row,fp_col=column}

getPosition`			:: Simple_0_1 Position e
getPosition`= \continue state=:{input_name,row} = continue state $ LinePos input_name row

getLineAndColumn			:: Simple_0_1 LineAndColumn e
getLineAndColumn= \continue state=:{column,row} = continue state {lc_line=row, lc_column=column}

getFileName			:: Simple_0_1 String e
getFileName= \continue state=:{input_name} = continue state input_name

// parts are in the same line
toSafeInLine		:: !(Try_parser a b) -> Try_parser a b
toSafeInLine parser= f
where
	f success fail state=:{column} = parser success (fail o setColumn) state
	where
		setColumn state= {state& column=column}

toSafe		:: !(Try_parser a b) -> Try_parser a b
toSafe parser= f
where
	f success fail state=:{column,row=oldRow,input}
		# (filePosition, input)= fposition input
		= parser success (fail o set filePosition) {state& input= input} with

			set filePosition state=:{row}
				| oldRow==row	= {state&column=column}
				#! (ok,input)= fseek state.input filePosition FSeekSet //<<- ("setPos",state.row,state.column,fp.cst_row,fp.cst_col)
				| not ok= abort "setPos"
				#! (line,input)= freadline input
				#! line= convertLine state.tabsize line
				= {state& line=line, row=oldRow, input=input, column=column}

///////////////////////////////////

nextLine :: Simple_0_0 e
nextLine = modifyStateWith nl
where
	nl state=:{input,tabsize}
		#! (line,input)= freadline input
		#! line= convertLine tabsize line
		= renewSemicolon {state& line=line, row=inc state.row, input=input, column=0}

convertLine :: !Int !String -> String
convertLine tabSize line
	| line==""	= "\0"
	# line= preProcess line
	# line= removeTabs tabSize line
	= line
where
	removeTabs tabSize line
		# size` = 1+ s 0 0
		# l= copy (createArray size` ' ') 0 0
		= {l& [size`-1]='\n'}
	where
		s i j
			| i==size line = j
			= case line.[i] of
				'\xD'	= j
				'\xA'	= j
				'\t'	= s (inc i) (tab j)
				_		= s (inc i) (inc j)
	
		copy :: !*String !Int !Int -> *String
		copy m i j
			| i==size line	= m
			= case line.[i] of
				'\xD'	= m
				'\xA'	= m
				'\t'	= copy m (inc i) (tab j)
				_		= copy {m& [j]=line.[i]} (inc i) (inc j)
		
		tab column = tabSize*(column/tabSize+1)
	
	changes :: [(!String,!String)]
	changes =:
		[	("/*2.0", "")
		,	("0.2*/", "")
		,	("//1.3", "/*")
		,	("//3.1", "*/")
			]
	
	preProcess line =
		f line changes
	where
		f line [] = line
		f line [(pattern,replacement):tl]
			| line === pattern			= replacement		//= replacement +++ line%(size replacement, size line-1)
			= f line tl
		where
			(===) line s= size line>=size s && compare 0
			where
				compare i= i==size s || line.[i]==s.[i] && compare (inc i)
	
///////////////////////////////

oneOrMore` f success fail state=:{line,column}
	= if (f line.[column]) (success (skipCharsBy f {state&column=inc column})) (fail state)

skipCharsBy cond state :== {state&column= f state.line state.column}
where
	f line i	= if (cond line.[i]) (f line (inc i)) i

skipSpace :: Simple_0_0 e
skipSpace =
	modifyStateWith (skipCharsBy ((==)' '))	&>
	(	(	(	checkEndOfLine
			<|>	parseString "//"
			)	&>
			nextLine
		<|>	parseString "/*"	&>
			skipComment
		<|>	check "*/"		 	&>
			skipChar 2			&>
			error "Unexpected end of comment."
		) &>
		skipSpace
	|>	epsilon
	)
where
	check s success fail state=:{line,column,context} = if (keyword` context s line column) success fail state

skipComment :: Simple_0_0 e
skipComment =
	modifyStateWith (skipCharsBy (\c=c<>'/'&&c<>'*'&&c<>'\n'&&c<>'\0')) &>
	(	parseString "*/"
	|>	checkFileEnd	&>
		error "End of file inside comment"
	|>	(	(	checkEndOfLine
			<|>	parseString "//"
			)	&>
			nextLine
		|>	parseString "/*"			&>
			skipComment
		|>	skipChar 1
		) &>
		skipComment
	)

basicToken :: Try_0_1 BasicValue e
basicToken =
	offsideChecked <&>
	(\f success fail state=:{column}= f success (drop_1 fail) state column)
	(	(	string '\"' &>
			end BVS
		<|>	string '\'' &>
			end BVC
		<|>	toSafeInLine (plusMinus <&> number)
		<|>	number
		) &>
		skipSpace
	)
where
	end f continue state=:{line,column} beg = continue state (f (line%(beg,column-1)))

	plusMinus success fail state=:{line,column}
		| column>0 && f line.[column-1] && g line.[column] = skipChar 1 success state
		= fail state
	where
		f ',' = True
		f '(' = True
		f ' ' = True
		f _ = False

		g '+' = True
		g '-' = True
		g _ = False

	number =
			parseChar '0'	&>
			(	(	oneOrMore` isOctDigit
				<|>	parseChar 'x' &>
					(	oneOrMore` isHexDigit
					|>	error "xxx"
					)
				) &>
				end	(BVInt o toInt)
			|>	realTail
			)
		<|>	oneOrMore` isDigit &>
			realTail
	where
		realTail =
				toSafeInLine
				(	parseChar '.' <&>
					oneOrMore` isDigit
				) &>
				(	parseChar 'E' &>
					(	parseChar '-'
					|>	parseChar '+'
					|>	epsilon
					) &>
					(	oneOrMore` isDigit
					|>	error ""
					)
				|>	epsilon
				) &>
				end BVR
			|>	end (BVInt o toInt)

	string char =
		parseChar char &>	g
	where
		g =
			modifyStateWith (skipCharsBy (\c=c<>char&&c<>'\\'&&c<>'\n'&&c<>'\0')) &>
			(	parseChar char
			|>	checkEndOfLine	&>
				error "End of line inside String."
			|>	checkFileEnd	&>
				error "End of file inside String."
			|>	(	escapeSeq
				|>	skipChar 1
				) &>
				g
			)

	escapeSeq =
		parseChar '\\' &>
		(	parseCharBy (\char= isMember char ['\\\'\"tnrfb'])
		|>	oneOrMore` isOctDigit
		|>	parseChar 'x' &>
			(	oneOrMore` isHexDigit
			|>	error "wrong sequense"
			)
		|>	error "wrong sequense"
		)

checkRes table :== f where
	f success fail state=:{line,column,context}
		= if (g 0) success fail state
	where
		g i	= i==size l || not (keyword` context l.[i] line column`) && g (inc i)

		l= table.[toInt line.[column]]
		column` = inc column

checkReserved = f where
	f success fail state=:{line,column,context}
		= if (g 0) success fail state
	where
		g i	= i==size l || not (keyword` context l.[i] line column`) && g (inc i)

		l= (if (context==NormalContext) reservedKW reservedKWType).[toInt line.[column]]
		column` = inc column

checkNoSC = checkRes noSemicolon

noSemicolon =: makeET
	["where","with","=",":==","=>",")","\0"]

reservedKWType =: makeET
	["=",":==",".","|","->","=>",":","..","special","where","&"] //"<="

reservedKW =: makeET
	["=",":==","=:","#","#!",".","|","\\","&","->","=>",":","\\\\","<-","<-:","<|-",".."
	,"where","with","let","in","case","class","instance","of","from","import","code","inline","_"]

makeET :: [String] -> {#{#String}}
makeET listWith = {(f o toChar) char\\ char<-[0..128]}
where
	f char = {k%(1,size k-1) \\ k<-listWith | k.[0]==char}

constructorToken :: Try_0_1 String e
constructorToken =
	offsideChecked			<&>
	(	checkCharBy isUpper
	<|>	isSpecialChar`		<&>
		flip checkDoubleColon
	)	<&>
	checkReserved			&>
	skipToTokenEnd			&>
	skipSpace
where
	isSpecialChar` success fail state=:{line,column,context} = if (isSpecialChar`` context line.[column]) success fail state

checkDoubleColon success fail state=:{line,column} = if (keyword` TypeContext` "::" line column) success fail state

doubleColonToken :: Try_0_0 e
doubleColonToken = checkDoubleColon &> skipChar 2 &> typeContext &> skipSpace
//success fail= typeContext (keyword "::" success (normalContext fail))


variableToken :: Try_0_1 String e
variableToken =
	offsideChecked								<&>
	checkCharBy (\c=isLower c||c=='_'||c=='`')	<&>
	checkReserved		&>
	skipToTokenEnd 		&>
	skipSpace			&>
	checkUnderscore
where
	checkUnderscore continue state=:{underscoreAllowed} variable
		| underscoreAllowed || variable.[0]<>'_' || size variable==1 = continue state variable
		= error "underscore variable not allowed" continue state variable

instance toString BasicValue
where
	toString x = case x of
		BVI s	= s
		BVInt i	= toString i
		BVC s	= s
		BVB b	= if b "True" "False"
		BVR s	= s
		BVS s	= s


generalToken :: Simple_0_1 String e
generalToken
	=	checkEndGroupToken	&> apply_0 "}"
	|>	semicolonToken		&> apply_0 ";"
	|>	basicToken			&> apply_1 toString
	|>	skipToTokenEnd				&> apply_1 toPrintable
where
	toPrintable string
		| size string==1 && not (isPrint string.[0]) = "ASCIICode: "+++(toString(toInt string.[0]))
		= string

codeLine			:: Try_0_1 String e		// anything but }
codeLine =
	flip (checkString "}")	&>
	getL					&>
	nextLine				&>
	skipSpace
where
	getL continue state=:{line,column}
		# end= findrArrElt (not o isSpace) line (size line-1)
		= continue state $ line%(column,end)

//////////////////////////

zeroOrMore :: !(Try_0_1 a e) -> Simple_0_1 [a] e
zeroOrMore f = checkError &> epsilon &> apply_0 [] |> cyc [] where
	cyc acc success=	f (checkError (drop_1 bb) (flip aa)) (flip success (reverse acc)) where
		aa x= cyc [x:acc] success
		bb	= cyc acc success o clearError

listWith :: !(Simple_0_1 a e) !(Try_0_0 e) !(Try_0_0 e) -> Simple_0_1 [a] e
listWith f comma close = checkError &> epsilon &> apply_0 [] |> g where
	g continue = cyc [] where
		cyc acc= f $ checkError (drop_1 (bb acc)) (flip \x= cc [x:acc])
		cc acc= comma (cyc acc) (flip continue (reverse acc))
		bb acc= skipTo close (cc acc) o clearError

	skipTo f continue = g
	where
		g = (f <|> checkFileEnd) continue (generalToken (drop_1 g))

groupWith			:: !(Simple_0_1 a e) !(Try_0_0 e) -> Simple_0_1 [a] e
groupWith f g =
	\continue=
 		beginGroupToken \old_offside=
 		    listWith f semicolonToken (checkEndGroupToken <|> g)
 	            (continue o setOffside old_offside)

checkEndGroupToken :: Try_0_0 e
checkEndGroupToken = checkColumnWith (<) <|> checkFileEnd

checkDefinitionEnd :: Try_0_0 e
checkDefinitionEnd = checkSemicolonToken <|> checkKeyword ';' <|> checkEndGroupToken

semicolonToken :: Try_0_0 e
semicolonToken = (checkSemicolonToken <|> keyword ';') &> modifyStateWith dropSemicolon

checkSemicolonToken = checkColumnWith (==) <&> checkNoSC

offsideChecked = checkColumnWith (>) <|> flip (checkSemicolonToken <|> checkEndGroupToken) &> modifyStateWith dropSemicolon

//////////////////////////

instance Keyword String
where
	keyword s = checkKeyword s &> skipChar (size s) &> skipSpace

	checkKeyword s = offsideChecked <&> check
	where
		check success fail state=:{line,column,context} = if (keyword` context s line column) success fail state

	expected e =
		flip checkError &> modifyStateWith setError &> getFilePosition &> apply_0 e &> generalToken &> writeErr1 |> epsilon

instance Keyword Char
where
	keyword char = checkKeyword char &> skipChar 1 &> skipSpace
//	keyword char = keyword (toString char)

//	checkKeyword char = checkKeyword (toString char)
	checkKeyword char = offsideChecked <&> check
	where
		check success fail state=:{line,column} = if (line.[column]==char) success fail state

	expected char = expected (toString char)

parseString s = checkString s &> skipChar (size s)
checkString s success fail state=:{line,column} = if (compare column 0) success fail state
where
	compare i j= j==size s	|| line.[i]==s.[j] && compare (inc i) (inc j)

/////////////////////////////

//apply_0 a success state :== success state a

apply_0 :: !(a) -> Simple_parser (a->*e) e
apply_0 f = \success state= success state (f)

apply_1 :: !(x->a) -> Simple_parser (a->*e) (x->*e)
apply_1 f = \success state x= success state (f x)

apply_2 :: !(y x->a) -> Simple_parser (a->*e) (x->*(y->*e))
apply_2 f = \success state x y= success state (f y x)

apply_3 :: !(z y x->a) -> Simple_parser (a->*e) (x->*(y->*(z->*e)))
apply_3 f = \success state x y z= success state (f z y x)

apply_4 :: !(v z y x->a) -> Simple_parser (a->*e) (x->*(y->*(z->*(v->*e))))
apply_4 f = \success state x y z v= success state (f v z y x)

apply_5 :: !(w v z y x->a) -> Simple_parser (a->*e) (x->*(y->*(z->*(v->*(w->*e)))))
apply_5 f = \success state x y z v w= success state (f w v z y x)

apply_6 :: !(l w v z y x->a) -> Simple_parser (a->*e) (x->*(y->*(z->*(v->*(w->*(l->*e))))))
apply_6 f = \success state x y z v w l= success state (f l w v z y x)

////////////////////////////////

checkFileEnd :: Try_0_0 e
checkFileEnd = checkCharBy ((==)'\0')

checkEndOfLine :: Try_0_0 e
checkEndOfLine = checkCharBy ((==)'\n')

parseChar char = parseCharBy ((==)char)
parseCharBy cond :== checkCharBy cond &> skipChar 1

checkCharBy cond :== \success fail state=:{line,column} = if (cond line.[column]) success fail state


createCharTable :: ![String] -> {#Int}
createCharTable listWith = {f (toChar char) \\ char<-[0..255]}
where
	f char	= foldl g 0 (zip2 [0..] listWith)
	where
		g val (x,s)
			#i = findlArrElt ((==)char) s 0
			| i==size s	= val
			= (1<<x) bitor val

charTable :: {#Int}
charTable =: createCharTable
	[ "abcdefghijklmnopqrstuvwxyz_`ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"	// 1
	, "~@#$%^?!+-*<>\\/|&=:."			// 2
	, "~@$%?+-<>#\\/|^&=:"				// 4
	]

NormalContext :== 3
TypeContext` :== 5

charType context char :== context bitand charTable.[toInt char]

skipToTokenEnd continue state=:{line,column,context}
	# col= tokenEnd context line column
	= continue {state& column= col} (line%(column,col-1))
where
	tokenEnd :: !Int !String !Int -> Int
	tokenEnd context line column
		# t= charType context line.[column]
		| t==0 = inc column
		= f (inc column)
	with
		f column
			| t<>charType context line.[column] = column
			= f (inc column)

keyword` ::	!Int !String !String !Int -> Bool
keyword` context s line column = compare column 0
where
	compare i j
		| j==size s
			# t= charType context line.[i-1]
			= t==0 || t<> charType context line.[i]
		= line.[i]==s.[j] && compare (inc i) (inc j)

isSpecialChar``		:: !Int !Char -> Bool
isSpecialChar`` context char		= charType context char > 1

////////////////////////////////


writeErr1 continue state=:{input_name} token message position = continue (writeMessage h state) where
	h error_file = error_file
			<<< "Parse error ["
			<<< input_name <<< ','
			<<< position 
			<<< "]: "
			<<< message
		  	<<< " expected instead of "
		  	<<< token <<< "\n"

error				:: !String -> Simple_0_0 e
error message
	=	flip checkError		&>
		modifyStateWith setError		&>
		getFilePosition		&>
		f
	|>	epsilon
where
	f continue state=:{input_name} position = continue (writeMessage h state)
	where
		h x= x	<<< "Parse error ["
				<<< input_name <<< ','
				<<< position 
				<<< "]: " <<< message <<< "\n"

warning :: !String -> Simple_0_0 e
warning message
	=	flip checkError		&>
		modifyStateWith setError		&>
		getFilePosition		&>
		f
	|>	epsilon
where
	f continue state=:{input_name} position = continue (writeMessage h state)
	where
		h x= x	<<< "Parse warning ["
				<<< input_name <<< ','
				<<< position 
				<<< "]: " <<< message <<< "\n"
				
				
//////////// original

fopenInSearchPaths :: !{#Char} SearchPaths !Int (ModTimeFunction *f) !*f -> (Optional (*File, {#Char}),!*f) | FileSystem f
fopenInSearchPaths fileName searchPaths mode modtimefunction f
	# filtered_locations
		=	filter (\(moduleName,path) -> moduleName == fileName) searchPaths.sp_locations
	| isEmpty filtered_locations
		=	fopenAnywhereInSearchPaths fileName searchPaths.sp_paths mode modtimefunction f
	# (_, path)
		=	hd filtered_locations
	# (opened, file, f)
		=	fopen (path + fileName) mode f
	| opened
		=	getModificationTime file path modtimefunction f
	| otherwise
		=	(No, f)
	where
		fopenAnywhereInSearchPaths :: !{#Char} ![{#Char}] !Int (ModTimeFunction *f) *f -> (Optional (*File, {#Char}),!*f) | FileSystem f
		fopenAnywhereInSearchPaths fileName [] _ _ f
				=	(No, f)
		fopenAnywhereInSearchPaths fileName [path : paths] mode modtimefunction f
			# (opened, file, f)
				=	fopen (path + fileName) mode f
			| opened
				=	getModificationTime file path modtimefunction f
			// otherwise
				=	fopenAnywhereInSearchPaths fileName paths mode modtimefunction f

		getModificationTime :: *File {#Char} (ModTimeFunction *f) *f -> (Optional (*File, {#Char}),!*f) | FileSystem f
		getModificationTime file path modtimefunction f
			# (time, f)
				=	modtimefunction (path + fileName) f
			=	(Yes (file, time), f)


