implementation module parse

import StdEnv, RWSDebug
import combinators
import syntax, utilities, predef, containers

//////////////

listWith`` str f g		:== listWith f (keyword str) (checkKeyword str <|> g)
listWith` str f str2	:== listWith f (keyword str) (checkKeyword str <|> checkKeyword str2) &> keyword` str2

keyword` s			:== keyword s |> expected s

(.:) infixl 5
(.:) f g :== \continue= f (\state= continue state g) //f &> apply_0 g
(..:) infixl 5
(..:) f g :== \success = f success (\state= success state g)

(!?) infixl 5
(!?) f s :== f |> expected s

(!?:) infixl 5
(!?:) f s = f |> (expected s .: s)
(&:>) infixr 4
(&:>) f g :== \success= f (\state a= g a success state)

oneElem f g l = case l of
	[x]	= f x
	x	= g x

oneElem` success fail state l = case l of
	[x]	= success state x
	x	= fail state x

//////////////////////

identToken	:: Try_0_1 String e
identToken	= variableToken <|> constructorToken

wantModule :: !Bool !Ident !Position !Bool !*HashTable !*File !SearchPaths (ModTimeFunction *Files) !*Files
	-> (!Bool, !ParsedModule, !*HashTable, !*File, !*Files)
wantModule iclmodule file_id import_file_position support_generics hash_table error searchPaths modtimefunction files
	= wantModule` iclmodule file_id import_file_position support_generics hash_table error searchPaths modtimefunction files 
		(header file_id iclmodule)

////////////////////////

header :: !Ident !Bool -> Simple_0_1 ParsedModule e
header file_id=:{id_name} iclmodule =
	if iclmodule
	(	keyword "implementation"	.: MK_Module |>
		keyword "system"			.: MK_System |>
		epsilon						.: MK_Main
	)
	(	keyword "system"			.: MK_System |>
		keyword` "definition"		.: MK_Module
	) &>
	keyword` "module"					&>
	keyword` id_name					&>
	setLayoutTrue						&>
	groupWith
	(	parsedDefinition &>
		checkDefinition
	) succeed &>
	apply_1 removeLastErr				&>
	if iclmodule
		(module_id id_name &> swap &> apply_2 cons) 
		epsilon	&>
	checkFileEnd !? "end of file"		&>
	apply_2	\mod_type defs
			= {mod_name= file_id, mod_modification_time= "", mod_type= mod_type, mod_imports= [], mod_imported_objects = [], mod_defs= defs}
where
	removeLastErr [] = []
	removeLastErr [PD_Erroneous] = []
	removeLastErr [a:b] = [a:removeLastErr b]

	checkDefinition = if iclmodule
		(check "implDef" isImplDef)
		(check "defDef" isDefDef)
	where
		isDefDef (PD_Function _ _ _ _ _ t) = case t of FK_Macro= True; _= False
		isDefDef _ = True

		isImplDef _ = True

cons hd tl :== [hd:tl]

parsedDefinition :: Simple_0_1 ParsedDefinition e
parsedDefinition =
	getPosition` &>
	(	typeDef
	|>	functionDef
	|>	importDef
	|>	instanceDef
	|>	classDef
	|>	checkFileEnd !? "definition"	&>
		drop_1							&>
		apply_0 PD_Erroneous
	)

existential :: Simple_0_1 [ATypeVar] e
existential =
		keyword "E." &>
		oneOrMore "existential quantified variable" aTypeVar &>
		keyword` ":"
	|>	apply_0 []

universal :: Simple_0_1 [ATypeVar] e
universal =
		keyword "A." &>
		oneOrMore "universal quantified variable" aTypeVar &>
		keyword` ":"
	|>	apply_0 []

field :: !Ident -> Simple_0_1 ParsedSelector e
field record_type =
	variableToken !?: "var"			&>
	toIdent`` (IC_Field record_type) &>
	toIdent`` IC_Selector			&>
	into_hashTable IC_Expression	&>
	getPosition`					&>
	doubleColonToken !? "::"		&>
	sAType`							&>
	normalContext					&>
	apply_5	\field_name selector_name field_var pos {s_annotation,s_type} =
			{	ps_field_name = field_name, ps_selector_name = selector_name, ps_field_type = s_type,
				ps_field_annotation = s_annotation,
				ps_field_var = field_var, ps_field_pos = pos}


typeDef :: Try_1_1 Position ParsedDefinition e
typeDef =
	doubleColonToken	&>
	sAType`				&>
	check "type constructor" isConstructor &>
	contexts			&>
	(	keyword ":=="	&>
		sAType`			&>
		warnIfStrict	&>
		apply_4 c_syn
	|>	keyword "="		&>
		existential		&>
		(	keyword '{' &>
			record
		|>	constructor		&>
			(	keyword "|" &>
				listWith`` "|" (existential &> constructor) checkDefinitionEnd
			|>	apply_0 []
			) &>
			apply_2 cons &>
			apply_4 c_algeb
		)
	|>	apply_3 c_abstr
	) &>
	normalContext
where
	isConstructor {s_type={at_type=TA _ _}} = True
	isConstructor _ = False

	record continue state exi con x=:{s_type={at_type=TA {type_name=type_name=:{id_name}} _}} pos
		# (name,state)= stringToIdent ("_"+++id_name) IC_Expression state
		= f type_name continue state name exi con x pos
	where
		f tname=
			(	keyword ",}" &>
				groupWith (field tname) succeed
			|>	listWith` ',' (field tname) '}'
			) &>
			apply_6 c_rec

	c_rec pos {s_type={at_attribute,at_type=TA {type_name,type_arity} args}} contexts exi rec_cons_ident fields
		= PD_Type
			{	td_name = type_name, td_index = -1, td_arity = type_arity, td_args =map toTypeVar args, td_attrs = [], td_attribute = at_attribute, td_context = contexts,
				td_pos = pos, td_rhs = SelectorList rec_cons_ident exi fields, td_used_types = [] }

	c_algeb pos {s_type={at_attribute,at_type=TA {type_name,type_arity} args}} contexts defs
		= PD_Type
			{	td_name = type_name, td_index = -1, td_arity = type_arity, td_args =map toTypeVar args, td_attrs = [], td_attribute = at_attribute, td_context = contexts,
				td_pos = pos, td_rhs = ConsList defs, td_used_types = [] }

	c_syn pos {s_type={at_attribute,at_type=TA {type_name,type_arity} args}} contexts atype
//		# (at_attribute, properties) = determine_properties s_annotation at_attribute
		= PD_Type
			{	td_name = type_name, td_index = -1, td_arity = type_arity, td_args =map toTypeVar args, td_attrs = [], td_attribute = at_attribute, td_context = contexts,
				td_pos = pos, td_rhs = TypeSpec atype, td_used_types = [] }

	c_abstr pos {s_annotation,s_type={at_attribute,at_type=TA {type_name,type_arity} args}} contexts
//			| td_attribute == TA_Anonymous || td_attribute == TA_Unique || td_attribute == TA_None
		# (at_attribute, properties) = determine_properties s_annotation at_attribute
		= PD_Type
			{	td_name = type_name, td_index = -1, td_arity = type_arity, td_args =map toTypeVar args, td_attrs = [], td_attribute = at_attribute, td_context = contexts,
				td_pos = pos, td_rhs = EmptyRhs properties, td_used_types = [] }
	where
		determine_properties :: !Annotation !TypeAttribute -> (!TypeAttribute, !BITVECT)
		determine_properties annot attr
			| annot == AN_Strict
				| attr == TA_Anonymous
					= (TA_None, cIsHyperStrict)
					= (attr, cIsHyperStrict bitor cIsNonCoercible)
			| attr == TA_Anonymous
				= (TA_None, cAllBitsClear)
				= (attr, cIsNonCoercible)

	toTypeVar {at_attribute,at_type=TV x} = {atv_attribute=at_attribute, atv_variable= x}
	toTypeVar {at_attribute} = {atv_attribute=at_attribute, atv_variable= MakeTypeVar erroneousIdent}

//constructor :: Simple_0_1 ParsedConstructor e
constructor =
	(	keyword '('						&>
		constructorToken`	&>
		into_hashTable IC_Expression	&>
		keyword` ')'					&>
		priorityToken
	|>	constructorToken`	&>
		into_hashTable IC_Expression .: NoPrio
	)				&>
	getPosition`	&>
	zeroOrMore brackSAType &>
	apply_5 \exi pc_cons_name prio pc_cons_pos pc_arg_types =
		{	pc_cons_name = pc_cons_name, pc_arg_types = atypes_from_satypes pc_arg_types, pc_args_strictness=strictness_from_satypes pc_arg_types, pc_cons_arity = length pc_arg_types,
	  		pc_cons_prio = prio, pc_exi_vars = exi, pc_cons_pos = pc_cons_pos}

className =
	identToken !?: "class name" &>
	into_hashTable IC_Class

classDef :: Try_1_1 Position ParsedDefinition e
classDef =
	keyword "class"		&>
	(	keyword '(' 	&>
		className		&>
		keyword` ')'	&>
		priorityToken
	|>	className		&>
		apply_0 NoPrio
	)					&>
	zeroOrMore aTypeVar &>
	apply_1 (\args= convert_class_variables args 0 0) &>
	(	doubleColonToken .: []	&>
		apply_5 c_class			&>
		symbolType				&>
		normalContext			&>
		c_members
	|>	contexts				&>
		apply_5 c_class 		&>
		apply_1 fst3			&>
		(	keyword "where" 	&>
			groupWith (parsedDefinition /**/) succeed
		|>	apply_0 []
		)
	) &>
	apply_2 PD_Class
where
	c_members continue pState tspec (class_def=:{class_name={id_name}},prio,pos)
		# (member_id, pState) = stringToIdent id_name IC_Expression pState
		# members = [PD_TypeSpec pos member_id prio (Yes tspec) SP_None]
		= continue pState members class_def

	convert_class_variables [] arg_nr cons_vars
		= (arg_nr, [], cons_vars)
	convert_class_variables [{atv_attribute,atv_variable} : class_vars] arg_nr cons_vars
		# (arity, class_vars, cons_vars) = convert_class_variables class_vars (inc arg_nr) cons_vars
		= (arity, [atv_variable : class_vars], case atv_attribute of
			TA_None = cons_vars
			_ = cons_vars bitor (1 << arg_nr) ///
			)

	c_class pos class_id prio (class_arity, class_args, class_cons_vars) contexts
		= ({class_name = class_id, class_arity = class_arity, class_args = class_args,
	    	class_context = contexts, class_pos = pos, class_members = {}, class_cons_vars = class_cons_vars,
			class_dictionary = { ds_ident = { class_id & id_info = nilPtr }, ds_arity = 0, ds_index = NoIndex },
			class_arg_kinds = []}
		  , prio, pos)

contexts :: Simple_0_1 [TypeContext] e
contexts
	=	keyword "|" &>
		listWith`` "&" context succeed /*(checkDefinitionEnd <|> keyword "where" <|> keyword "special")*/ &>
		apply_1 flatten
	|>	apply_0 []
where
//	context :: Simple_0_1 [TypeContext] e
	context =
		listWith`` ',' className` succeed &>
		typeContext		&>
		oneOrMore "type"
		(	brackSAType &>
			removeSA
		) &>
		normalContext	&>
		c_context

	className` =
		className &>
		apply_1 \ident= TCClass {glob_object = MakeDefinedSymbol ident NoIndex (-1), glob_module = NoIndex }

	c_context continue pState types tc_classes
		# (c,pState)= mapSt (build_context types (length types)) tc_classes pState
		= continue pState c
	where
		build_context types length_types (TCClass class_global_ds=:{glob_object}) pState
			# tc_class = TCClass {class_global_ds & glob_object = {glob_object & ds_arity = length_types}}
			= ({ tc_class = tc_class, tc_var = nilPtr, tc_types = types}, pState)
		build_context types 1 (TCGeneric gtc=:{gtc_generic=gtc_generic=:{glob_object}}) pState
			# gtc = { gtc & gtc_generic = {gtc_generic & glob_object = {glob_object & ds_arity = 1}}} 
			= ({ tc_class = TCGeneric gtc, tc_var = nilPtr, tc_types = types }, pState)
		
//		build_context types length_types tc_class=:(TCGeneric _) pState
//			# pState = parseErrorSimple "type context" "generic class can have only one class argument" pState					
//			= (abort "No TypeContext", pState)	

instanceDef :: Try_1_1 Position ParsedDefinition e
instanceDef =
	keyword "instance"			&>
	identToken !?: "class name" &>
	into_hashTable IC_Class			&:> \class_id=:{id_name} =
	listWith`` ','
		(	apply_0 id_name			&>
			tc
		)
		checkDefinitionEnd				&>
	(	keyword "where"			&>
		groupWith 
			parsedDefinition
			succeed
	|>	apply_0 []
	)	&>
	apply_3 (c_inst class_id)
where
	c_inst class_id pos [t] members= PD_Instance (cc class_id pos t members)
	c_inst class_id pos types []= PD_Instances [cc class_id pos t [] \\ t<-types]

	cc class_id pos (id,types,context) mem=
		{	pi_class = class_id, pi_ident = id
		,	pi_types = types, pi_context = context
		,	pi_members = mem, pi_specials = SP_None/*specials*/
		,	pi_pos = pos}

	tc =
		typeContext					&>
		oneOrMore "ss" (brackSAType &> removeSA) &:> \types=
		into_hashTable (IC_Instance types) &>
		normalContext				&>
		contexts					&>
		normalContext				&>
		apply_2 \id context= (id,types,context)
	
sAType` :: Simple_0_1 SAType e
sAType` =
	sAType |>
	expected "type" .: defa2

sAType :: Try_0_1 SAType e
sAType =
//	universal	&>
	brackSAType &>
	zeroOrMore (brackSAType &> warnIfStrict) &>
	(	keyword "->"	&>
		sAType` 		&>
		warnIfStrict	&>
		apply_3 (\type types t= {type& s_type=make_curry_type [type.s_type:types] t})
	|>	convert
	)
where
	make_curry_type [t1] res_type
		= {at_attribute = TA_None, at_type = t1 --> res_type}
	make_curry_type [t1:tr] res_type
		= {at_attribute = TA_None, at_type = t1 --> make_curry_type tr res_type}
	make_curry_type _ _ = abort "make_curry_type: wrong assumption"

convert continue state [] type = continue state type
convert continue state types type 
	= case type.s_type.at_type of
		TA sym []	= c (TA { sym & type_arity = length types } types)
		TV tv		= c (CV tv :@: types)
		TArrow		= case types of
			[type1, type2]	= c (type1 --> type2)
			[type1]			= c (TArrow1 type1)
		TArrow1 type1	= case types of
			[type2]	= c (type1 --> type2)
//		= (TE, parseError "Type" No "ordinary type variable" pState)
where
	c at_type = continue state {type& s_type={type.s_type&at_type=at_type}}

decons continue state [x:y] = continue state y x


funType :: Simple_0_2 [SAType] AType e
funType =
	oneOrMore "type" brackSAType	&>
	(	keyword "->"	&>
		sAType`
	|>	decons	&>
		apply_1 (map \{s_type}=s_type) &>
		convert			&>
		apply_0 []			&>
		swap
	)					&>
	warnIfStrict		&>
	normalContext

HsymbolType :: Simple_0_1 SymbolType e
HsymbolType =
	(/*	toSafe
		(	contexts	<&+>
			keyword "=>"
		)	&>
		funType		&>
		(\continue state a b c=continue state c a b)	&>
		apply_0 []
	+|*/	funType		&>
		contexts		&>
		typeContext		&>
		coercions
	) &>
	apply_4 \args result context attr_env =
		{	st_vars = [], st_args = atypes_from_satypes args
		,	st_args_strictness = strictness_from_satypes args
		,	st_arity = length args, st_result = result
		,	st_context = context, st_attr_env = attr_env, st_attr_vars = [] }

symbolType :: Simple_0_1 SymbolType e
symbolType =
	universal		&>
	drop_1			&>
	oneOrMore "type" brackSAType	&>
	(	keyword "->"	&>
		sAType`
	|>	decons	&>
		apply_1 (map \{s_type}=s_type) &>
		convert			&>
		apply_0 []			&>
		swap
	)					&>
	warnIfStrict		&>
	normalContext		&>
	contexts			&>
//	typeContext			&>
	coercions			&>
//	normalContext		&>
	apply_4 \args result context attr_env =
		{	st_vars = [], st_args = atypes_from_satypes args
		,	st_args_strictness = strictness_from_satypes args
		,	st_arity = length args, st_result = result
		,	st_context = context, st_attr_env = attr_env, st_attr_vars = [] }

warnIfStrict :: Simple_1_1 SAType AType e
warnIfStrict = \continue state {s_annotation,s_type} = case s_annotation of
	AN_None	= continue state s_type
	_		= warning "! removed" continue state s_type

removeSA =
	warnIfStrict &>
	removeAttr

removeAttr :: Simple_1_1 AType Type e
removeAttr = \continue state {at_attribute,at_type}= case at_attribute of
	TA_None	= continue state at_type
	_		= warning "attribute removed" continue state at_type

determAttr = \continue state {at_attribute,at_type} attr= case at_attribute of
	TA_None	= continue state at_type attr
	a		= case attr of
		TA_None	= continue state at_type a
		_		= error "more type attributes" continue state at_type attr
// parseError "simple type" No ("More type attributes, "+toString attr+" and "+toString at_attribute+", than")

check :: !String !(a->Bool) -> Simple_0_0 (a->*e)
check message cond= \continue state a= if (cond a) continue (expected message continue) state a

adjustAttr= \continue state name a = case a of
	TA_Anonymous =
		(	into_hashTable IC_TypeAttr			&>
			apply_1 makeAttributeVar		&>
			apply_1 TA_Var					&>
			swap
		) continue state name name
	attr			= continue state name attr

aTypeVar :: Try_0_1 ATypeVar e
aTypeVar =
	(	(	keyword '*' .: TA_Unique
		<|>	keyword '.'	.: TA_Anonymous
		) &>
		variableToken !?: "var"
	<|>	variableToken .: TA_None	&>
		swap
	)				&>
	adjustAttr		&>
	into_hashTable IC_Type &>
	apply_1 MakeTypeVar &>
	apply_2 \attr t= {atv_attribute = attr, atv_variable = t}
	
defa2={s_annotation=AN_None,s_type={at_attribute=TA_None,at_type=TE}}
	
//swap :: (S2 a b e) -> S2 b a e
swap continue state a b:== continue state b a

add x f success fail state = f success (drop_1 fail) state x

drop2 message x= \continue state a = if (x==a) continue (error (message+++" expected") continue) state

strict =
		keyword '!'
	|>	epsilon

brackSAType` =
		brackSAType
	|>	expected "type" .: defa2

brackSAType :: Try_0_1 SAType e
brackSAType =
	add AN_None (
	add TA_None (
	brackType &>
	apply_3 \ann a t= {s_annotation=ann,s_type={at_attribute= a, at_type= t}}
	))
where
//	brackType`` :: Simple_0_1 Type e
	brackType`` =
			brackType
		|>	expected "brType" .: TE

//	brackType :: Try_0_1 Type e
	brackType =
			keyword '!'				&>
			drop2 "no !" TA_None	&>
			drop2 "no !" AN_None .: AN_Strict .: TA_None &>
			brackType``
		<|>	keyword '.'				&>
			drop2 "no ." TA_None .: TA_Anonymous		&>
			brackType``
		<|>	keyword '*'				&>
			drop2 "no *" TA_None .: TA_Unique			&>
			brackType``
		<|>	keyword '{'				&>
			(	keyword "#" .: makeUnboxedArraySymbol
			|>	keyword '!' .: makeStrictArraySymbol
			|>	apply_0 makeLazyArraySymbol
			)						&>
			(	sAType				&>
				warnIfStrict		&>
				apply_1 (\x=[x])
			|>	apply_0 []
			)						&>
			keyword` '}'			&>
			apply_2 (\fun args= TA (fun (length args)) args)
		<|>	keyword '('				&>
			listWith` ',' sAType` ')'	&>
			(	oneElem`		&>
				warnIfStrict	&>
				determAttr
			|>	apply_1 \satypes= let arity= length satypes
						in TAS (makeTupleTypeSymbol arity arity) (map (\x=x.s_type) satypes) (strictness_from_satypes satypes)
			)
		<|>	(	keyword "[!" .: HeadStrict
			<|>	keyword "[#" .: HeadUnboxed
			<|>	keyword '['	.: HeadLazy
			) &>
			(	keyword ']' .: []
			|>	sAType`				&>
				warnIfStrict		&>
				apply_1 (\x=[x])	&>
				keyword` ']'
			)						&>
			apply_2 (\strict type= TA (makeListTypeSymbol strict (length type)) type)
		<|>	variableToken			&>
			(	keyword ":"			&>
				swap				&>
				drop2 "no :" TA_None	&>
				into_hashTable IC_TypeAttr	&>
				apply_1 makeAttributeVar	&>
				apply_1 TA_Var			&>
				brackType``
			|>	adjustAttr				&>
				into_hashTable IC_Type	&>
				apply_1 MakeTypeVar 	&>
				apply_1 TV
			)
		<|>	constructorToken			&>
			\continue state name = case name of
				"Int"		= continue state (TB BT_Int)
				"Char"		= continue state (TB BT_Char)
				"Real"		= continue state (TB BT_Real)
				"Bool"		= continue state (TB BT_Bool)
				"Dynamic"	= continue state (TB BT_Dynamic)
				"File"		= continue state (TB BT_File)
				"World"		= continue state (TB BT_World)
				"String"	= continue state makeStringType
				name
					# (name,state)= stringToIdent name IC_Type state
					= continue state (TA (MakeNewTypeSymbIdent name 0) [])


coercions :: Simple_0_1 [AttrInequality] e
coercions =
		keyword ',' &>
		keyword` '[' &>
		listWith` ',' inequality ']' &>
		apply_1 flatten
	|>	apply_0 []
where
	inequality =
		oneOrMore "var"
		(	variableToken &>
			into_hashTable IC_TypeAttr &>
			apply_1 makeAttributeVar
		) &>
		keyword` "<=" &>
		variableToken !?: "var" &>
		into_hashTable IC_TypeAttr 	&>
		apply_1 makeAttributeVar	&>
		apply_2 \off dem= [{ai_demanded= dem, ai_offered= offered} \\ offered <- off]

importDef :: Try_1_1 Position ParsedDefinition e
importDef =
		keyword "import"	&>
		drop_1				&>
		listWith`` ','
		(	getPosition`	&>
			moduleName		&>
			apply_0 []			&>
			apply_3 c_import
		) checkDefinitionEnd			&>
		apply_1 PD_Import
	<|>	keyword "from"		&>
		moduleName			&>
		keyword` "import"	&>
		listWith`` ','
			importDeclaration
			checkDefinitionEnd		&>
		apply_3 c_import		&>
		apply_1 \x= PD_Import [x]
where
	c_import pos mod_ident import_symbols
		= { import_module = mod_ident, import_symbols = import_symbols, import_file_position = pos }
	
	names f =
		keyword '(' &>
		(	keyword ".." &>
			keyword` ')' .: []
		|>	listWith` ',' (f &> apply_1 \id= {ii_ident= id, ii_extended= False}) ')'
		)

	importDeclaration =
			keyword "class" &>
			identToken !?: "class name" &>
			into_hashTable IC_Class &>
			(	names
				(	identToken !?: "id" &>
					into_hashTable IC_Expression
				) &>
				apply_1 Yes
			|>	apply_0 No
			) &>
			apply_2 (\name members= ID_Class { ii_ident = name, ii_extended = False /*ii_extended*/ } members)
		|>	keyword "instance" &>
			identToken !?: "class name" &>
			into_hashTable IC_Class &>
			typeContext &>
			oneOrMore "type" (sAType &> removeSA) &>
			normalContext &>
			contexts &> f
		|>	doubleColonToken &>
			identToken !?: "type constructor" &>
			into_hashTable IC_Type &>
			(	keyword '{' &>
				(	keyword ".." &>
					keyword` '}' .: []
				|>	listWith` ','
					(	variableToken !?: "var" &>
						into_hashTable IC_Expression &>
						apply_1 \id= {ii_ident= id, ii_extended= False}
					) '}'
				) &>
				apply_1 Yes .: ID_Record
			|>	names
				(	constructorToken` &>
					into_hashTable IC_Expression
				) &>
				apply_1 Yes .: ID_Type
			|>	apply_0 No .: ID_Type
			) &>
			apply_3 (\name memb c= c {ii_ident = name, ii_extended = False } memb) &> normalContext
		|>	identToken &>
			into_hashTable IC_Expression &>
			optional_extension &>
			apply_2 (\fun_id ii_extended= ID_Function {ii_ident = fun_id, ii_extended = ii_extended })
		|>	expected "decl" .: ID_OldSyntax []
	where
		optional_extension =
			keyword ".." .: True ..: False

		f continue state contexts types name=:{id_name}
			# (inst_id, state)= stringToIdent id_name (IC_Instance types) state
			= continue state (ID_Instance { ii_ident = name, ii_extended = False } inst_id (types,contexts))
	
	moduleName =
		underscoreAllowed
		(	identToken !?: "Module name" &>
			into_hashTable IC_Module
		)

specials :: Simple_0_1 Specials e
specials =
		keyword "special" &>
		groupWith (listWith`` ',' subs checkDefinitionEnd) succeed &>
		apply_1 SP_ParsedSubstitutions
	|>	apply_0 SP_None
where
	subs =
		variableToken !?: "typeVar" &>
		into_hashTable IC_Type &>
		apply_1 MakeTypeVar &>
		keyword` "="	&>
		brackSAType`	&>
		removeSA		&>
		apply_2 \type_var type= {bind_src = type, bind_dst = type_var}

functionDef :: Try_1_1 Position ParsedDefinition e
functionDef =
		(	identToken				&>
			into_hashTable IC_Expression	&>
			apply_0	NoPrio
		<|>	toSafe
			(	keyword '('			<&>
				identToken			<&+>
				keyword ')'
			)						&>
			into_hashTable IC_Expression	&>
			priorityToken
		)							&>
		(	doubleColonToken		&>
			symbolType				&>
			specials				&>
			normalContext			&>
			apply_5 (\pos name prio type specials=		PD_TypeSpec pos name prio (Yes type) specials)
		|>	checkDefinitionEnd				&>
			apply_3 (\pos name prio=					PD_TypeSpec pos name prio No SP_None)
		|>	zeroOrMore pattern		&>
			funRhs					&>
			apply_5 (\pos name prio lhs (funkind,rhs)=	PD_Function pos name (is_infix prio) lhs rhs funkind)
		)
	<|>	pattern						&>
		funRhs						&>
		apply_3 (\pos patt (sym,rhs)=					PD_NodeDef pos patt rhs)

priorityToken :: Simple_0_1 Priority e
priorityToken =
		fixity &>
		digitToken ..: 9 &>
		apply_2 Prio
	|>	apply_0 (Prio NoAssoc 9)
where
	digitToken :: Try_0_1 Int e
	digitToken=
		basicToken &>
		check "digit" isDigit &>
		apply_1 conv
	where
		isDigit (BVInt n) = n>=0 && n<=10 
		isDigit _ = False
		conv (BVInt x) = x
		conv _ = 9

	fixity :: Try_0_1 Assoc e
	fixity =
		keyword "infix"		.: NoAssoc		<|>
		keyword "infixr"	.: RightAssoc	<|>
		keyword "infixl"	.: LeftAssoc

endLD =
	checkKeyword ')'	<|>
	checkKeyword '}'	<|>
	checkKeyword ']'	<|>
	checkKeyword "in"

localDefs :: Simple_0_1 LocalDefs e
localDefs =
	groupWith (parsedDefinition &> isLD) endLD &>
	apply_1 LocalParsedDefs
where
	isLD continue state def = case def of
		PD_Function pos name False [] rhs (FK_Function _) =
			continue state $ PD_Function pos name False [] rhs FK_NodeDefOrFunction
		x = continue state x


letBefores :: Simple_0_1 [NodeDefWithLocals] e
letBefores =
	zeroOrMore letBefore &> apply_1 flatten

letBefore :: Try_0_1 [NodeDefWithLocals] e
letBefore =
		keyword "#" &>
		groupWith (letDef False) succeed
	<|>	keyword "#!" &>
		groupWith (letDef True) succeed
where
	letDef strict=
		graphPattern &>
		getPosition` &>
		keyword` "=" &>
		graphExpr		&>
		(	keyword "with" &>
			localDefs
		|>	apply_0 (LocalParsedDefs [])
		) &>
		apply_4 \lhs_exp pos rhs_exp locals=
			{	ndwl_strict= strict, ndwl_def= {bind_dst = lhs_exp, bind_src= rhs_exp}
			,	ndwl_locals= locals, ndwl_position= pos}

//funKind` :: Simple_0_1 String e
//funKind` = funKind |> expected "define" .: "="

funKind :: Try_0_1 String e
funKind =
	keyword`` "=:"	<|>
	keyword`` ":==" <|>
	keyword`` "=>"	<|>
	keyword`` "->"	<|>
	keyword`` "="
where
	keyword`` s = keyword s .: s

strToFunKind s = case s of
	"=:"	= FK_Caf
	":=="	= FK_Macro
	"=>"	= FK_Function cNameNotLocationDependent
	"->"	= FK_Function cNameNotLocationDependent
	"="		= FK_Function cNameNotLocationDependent

:: XX = LetB ![NodeDefWithLocals] | Guard !String !GuardedExpr


caseRhs :: Simple_0_1 (!FunKind,!Rhs) e
caseRhs = funRhs //(funKind |> keyword` "->" .: FK_Function cNameNotLocationDependent) &> graphExpr &> apply_1 (\x= UnGuardedExpr (c_ewl [] x)) &> apply_2 \fKind rhs= (fKind, {rhs_alts= rhs, rhs_locals= LocalParsedDefs []})


funRhs :: Simple_0_1 (!FunKind,!Rhs) e
funRhs =
	body &>
	(	keyword "where" &>
		localDefs
	|>	apply_0 (LocalParsedDefs [])
	) &>
	apply_2 \(fKind,rhs) locals= (strToFunKind fKind, {rhs_alts= rhs, rhs_locals= locals})

body :: Simple_0_1 (!String,!OptGuardedAlts) e
body =
	alts &>
	apply_1 (sort "" [] []) &>
	(	funKind &>
		graphExpr &>
		apply_3 make1
	|>	apply_1 make2
	)
where
//		sort :: ![GuardedExpr] ![NodeDefWithLocals] !XX -> OptGuardedAlts
	sort fk g lb [LetB x :tl] = sort fk g (lb++x) tl
	sort fk` g lb [Guard fk x :tl]
		| fk`<>"" && fk<>fk`	= abort "....."
		# x= {x& alt_nodes= lb}
		= sort fk [x:g] [] tl
	sort fk g lb [] = (fk, reverse g, lb)

make1 (fk`,[],lb) fk x
	| fk`<>"" && fk<>fk`	= abort "....."
	= (fk, UnGuardedExpr (c_ewl lb x))
make1 (fk`,g,lb) fk x
	| fk`<>"" && fk<>fk`	= abort "....."
	= (fk, GuardedAlts g (Yes (c_ewl lb x)))
make2 (fk,g,[]) = (fk, GuardedAlts g No)
make2 (fk,g,_) = (fk, GuardedAlts g No)  <<- " __err__ "

alts =
	zeroOrMore (letBefore &> apply_1 LetB <|> guard)

//	guard :: Try_0_1 (!FunKind,!GuardedExpr) e
guard =
	keyword "|" &>
	(keyword "otherwise" .: true_ |> graphExpr) &>
	body &>
	apply_2 (\g (fk,b)= Guard fk (c_alt g b))

true_= PE_Basic (BVB True)

c_alt guard body
	= { alt_nodes = [], alt_guard = guard, alt_expr = body,
		alt_ident = guard_ident 1, alt_position = NoPos }

guard_ident line_nr
	= { id_name = "_g;" +++ toString line_nr +++ ";", id_info = nilPtr }
	
c_ewl letB expr =
	{	ewl_nodes	= letB
	,	ewl_expr	= expr
	,	ewl_locals	= LocalParsedDefs []//:: !LocalDefs
	,	ewl_position= NoPos //pos
	}
	
is_infix NoPrio= False
is_infix _= True

//oneOrMore` :: !(Try_0_1 a e) -> Try_0_1 [a] e
//oneOrMore` f= \success fail state= zeroOrMore f (\state x= case x of []=fail state; x=success state x) state

oneOrMore :: !String !(Try_0_1 a e) -> Simple_0_1 [a] e
oneOrMore message f = zeroOrMore f &> h
where
	h continue state [] = expected message continue state []
	h continue state x = continue state x

graphPattern :: Simple_0_1 ParsedExpr e
graphPattern =
	oneOrMore "expression" pattern &>
	apply_1 (oneElem id PE_List)

pattern` :: Simple_0_1 ParsedExpr e
pattern` =
		pattern
	|>	expected "pattern" .: PE_Empty

pattern :: Try_0_1 ParsedExpr e
pattern =
		keyword '(' &>
		listWith` ',' graphPattern ')' &>
		apply_1 (oneElem (\x= case x of PE_Ident _= PE_List [x]; _= x) PE_Tuple)
	<|>	recOrArrayPatt
	<|>	listPatt
	<|>	keyword "_" .: PE_WildCard
	<|>	variableToken &>
		into_hashTable IC_Expression &>
		(	keyword "=:" &>
			pattern` &>
			apply_2 (\id expr= PE_Bound { bind_dst = id, bind_src = expr })
		|>	apply_1 PE_Ident
		)
	<|>	basicToken &>
		apply_1 PE_Basic
	<|>	constructorToken &>
		\continue state name = case name of
			"True"		= continue state (PE_Basic (BVB True))
			"False"		= continue state (PE_Basic (BVB False))
			name
				# (name,state)= stringToIdent name IC_Expression state
				= continue state (PE_Ident name)

recOrArrayPatt :: Try_0_1 ParsedExpr e
recOrArrayPatt =
	keyword '{' &>
	(	checkKeyword '[' &>
		listWith` ',' arrAssign '}' &>
		apply_1 PE_ArrayPattern
	|>	listWith` ',' fAss '}' &>
		apply_1 (PE_Record PE_Empty No)
	)
where
	fAss =
		variableToken !?: "selector" &>
		into_hashTable IC_Selector &>
		(keyword "=" &> graphPattern |> apply_0 PE_Empty) &>
		apply_2 \field_id field_expr= {bind_src= field_expr, bind_dst= field_id}

	arrAssign =
		keyword` '['	&>
		graphExpr		&>
		keyword` ']'	&>
		apply_1 (\x=[x])	&>
		keyword` "="	&>
		graphPattern	&>
		apply_2 \index_exprs pattern_exp= {bind_dst = index_exprs, bind_src = pattern_exp}

listPatt :: Try_0_1 ParsedExpr e
listPatt =
	keyword '[' &>
	strict	&>
	(	keyword ']' .: PE_Ident predefined_idents.[PD_NilSymbol]
	|>	listWith`` ',' graphPattern succeed &>
		(	keyword ":" &>
			graphPattern
		|>	apply_0 (PE_Ident predefined_idents.[PD_NilSymbol])
		) &>
		keyword` ']' &>
		apply_2 \l exp= foldr (\hd exp= PE_List [makeConsIdentExpression HeadLazy True,hd,exp]) exp l
	)

graphExpr :: Simple_0_1 ParsedExpr e
graphExpr =
	oneOrMore "expression" expression &>
	apply_1 (oneElem id PE_List)

expression` :: Simple_0_1 ParsedExpr e
expression` =
		expression
	|>	expected "expr" .: PE_Empty

constructorToken` = constructorToken !?: "constructor"
/*
		identToken &>
		check "constuctor" (\s=not (lower s.[0]))
	|>	expected "contructor" .: ""
where
	lower c = isLower c || c=='_' || c=='`'
*/
expression :: Try_0_1 ParsedExpr e
expression =
	(	//	keyword "()"	&>
		//	groupWith graphExpr epsilon
		keyword '('		&>
		listWith` ',' graphExpr ')' &>
		apply_1 (oneElem (\x= case x of PE_Ident _= PE_List [x]; _= x) PE_Tuple)
	<|>	recOrArrayExpr
	<|>	listExpr
	<|>	keyword "let"	&>
		localDefs		&>
		keyword` "in"	&>
		graphExpr		&>
		apply_2 (PE_Let False)
	<|>	keyword "case"	&>
		internIdent "_c" &>
		graphExpr		&>
		keyword` "of"	&>
		groupWith
		(	graphPattern &>
			caseRhs		&>
			apply_2 \pattern (kind,rhs)= { calt_pattern = pattern, calt_rhs = rhs }
		) endLD			&>
		apply_3 PE_Case
	<|>	keyword "if" 		&>
		internIdent "_if"	&>
		expression`			&>
		expression`			&>
		expression`			&>
		apply_4 PE_If
/*	<|>	keyword "if" 		>>
		internIdent` "_if"	>>= \id=
		expression``		>>= \cond=
		expression``		>>= \true=
		expression``		>>= \false=
		return (PE_If id cond true false)
*/
	<|>	keyword "\\" 	&>
		internIdent "\\" &>
		oneOrMore "pattern" pattern &>
		(keyword "->" |> keyword` "=") &>
		getPosition` 	&>
		graphExpr		&>
		apply_4 (\lam_ident lam_args pos exp= PE_Lambda lam_ident lam_args exp pos)
	<|>	keyword "code"	&>
		keyword "inline" .: True ..: False &>
		keyword` '{'	&>
		zeroOrMore codeLine &>
		keyword` '}'	&>
		swap			&>
		apply_2 PE_ABC_Code
	<|>	variableToken	&>
		into_hashTable IC_Expression &>
		apply_1 PE_Ident
	<|>	basicToken		&>
		apply_1 PE_Basic
	<|>	constructorToken &>
		\continue state name = case name of
			"True"		= continue state (PE_Basic (BVB True))
			"False"		= continue state (PE_Basic (BVB False))
			name
				# (name,state)= stringToIdent name IC_Expression state
				= continue state (PE_Ident name)
	) &>
	(	(	keyword "." .: ParsedNormalSelector
		<|>	keyword "!" .: (ParsedUniqueSelector False)
		)	&>
		swap			&>
		selectors		&>
		apply_3 PE_Selection
	|>	epsilon
	)

selectors :: Simple_0_1 [ParsedSelection] e
selectors =
	listWith`` "." selector succeed
where
	selector =
			keyword '['		&>
			graphExpr		&>
			keyword` ']'	&>
			apply_1 PS_Array
		|>	variableToken !?: "selector" &>
			into_hashTable IC_Selector .: No &>
			apply_2 PS_Record

recOrArrayExpr :: Try_0_1 ParsedExpr e
recOrArrayExpr =
	keyword '{' &>
	(	keyword '}' .: PE_ArrayDenot []
	|>	toSafe
		(	variableToken <&+>
			keyword "="
		) &>
		into_hashTable IC_Selector &>
		graphExpr &>
		apply_2 (\field_id field_expr= {bind_src= field_expr, bind_dst= field_id}) &>
		(	keyword ',' &>
			listWith` ','
			(	variableToken !?: "selector" &>
				into_hashTable IC_Selector &>
				keyword` "=" &>
				graphExpr &>
				apply_2 \field_id field_expr= {bind_src= field_expr, bind_dst= field_id}
			) '}' &>
			apply_2 cons &>
			apply_1 (PE_Record PE_Empty No)
		|>	keyword` '}' &>
			apply_1 (\x=[x]) &>
			apply_1 (PE_Record PE_Empty No)
		)
	|>	graphExpr &>
		(	keyword "&" &>
			getLineAndColumn &>
			listWith`` ','
			(	selectors &>
				keyword` "=" &>
				graphExpr &>
				apply_2 \selectors expr= {nu_selectors = selectors, nu_update_expr = expr}
			) succeed &>
			(	keyword "\\\\" &>
				listWith` ',' qualifier '}'
			|>	keyword` '}' .: []
			) &>
			c_arr
		|>	keyword "\\\\" &>
			listWith` ',' qualifier '}' &>
			apply_2 PE_ArrayCompr
		|>	keyword ',' &>
			listWith` ',' graphExpr '}' &>
			apply_2 cons &>
			apply_1 PE_ArrayDenot
		|>	keyword` '}' &>
			apply_1 (\a=[a]) &>
			apply_1 PE_ArrayDenot
		)
	)
where
	c_arr continue pState qualifiers updates position expr
		# type= No ///
		# (updatable_expr, pState) = test_qualifiers expr position qualifiers pState
		# (updated_expr, pState) = transform_record_or_array_update type updatable_expr updates 0 pState
		# res= add_qualifiers qualifiers expr updated_expr updatable_expr
		= continue pState res

	test_qualifiers :: !ParsedExpr !LineAndColumn [Qualifier] !State -> (!ParsedExpr, !State)
	test_qualifiers updateExpr _ [] pState
		=	(updateExpr, pState)
	test_qualifiers updateExpr {lc_line, lc_column} qualifiers pState
		# (ident, pState)
			=	stringToIdent ("a;" +++ toString lc_line +++ ";" +++ toString lc_column) IC_Expression pState
		=	(PE_Ident ident, pState)
	
	add_qualifiers :: ![Qualifier] !ParsedExpr !ParsedExpr !ParsedExpr -> ParsedExpr
	add_qualifiers [] _ update_expr _
		=	update_expr
	add_qualifiers qualifiers expr update_expr ident_expr
		=	PE_UpdateComprehension expr update_expr ident_expr qualifiers
	
	transform_record_or_array_update :: !(Optional Ident) ParsedExpr [NestedUpdate] !Int State -> (ParsedExpr, State)
	transform_record_or_array_update type expr updates level pState
		| is_record_update sortedUpdates
			=	transform_record_update type expr groupedUpdates level pState
		// otherwise
			=	transform_array_update expr updates level pState
		where
			sortedUpdates
				// sort updates by first field name, array updates last
				=	sortBy smaller_update updates
				where
					smaller_update :: NestedUpdate NestedUpdate -> Bool
					smaller_update a b
				 		=	smaller_selector (hd a.nu_selectors) (hd b.nu_selectors)
			 			where
							smaller_selector :: ParsedSelection ParsedSelection -> Bool
							smaller_selector (PS_Record ident1 _) (PS_Record ident2 _)
								=	ident1.id_name < ident2.id_name
							smaller_selector (PS_Record _ _) _
								=	True
							smaller_selector _ _
								=	False
	
			groupedUpdates
				// group nested updates by first field name
				=	groupBy equal_update sortedUpdates
				where
					equal_update :: NestedUpdate NestedUpdate -> Bool
					equal_update a b
						=	equal_selectors a.nu_selectors b.nu_selectors
			 			where
							equal_selectors :: [ParsedSelection] [ParsedSelection] -> Bool
							equal_selectors [PS_Record ident1 _ ,_ : _] [PS_Record ident2 _ ,_: _]
								=	ident1.id_name == ident2.id_name
							equal_selectors _ _
								=	False
		
			is_record_update [{nu_selectors=[select : _]} : _]
				=	is_record_select select
			is_record_update updates
				=	False
	
			is_record_select (PS_Record _ _)
				=	True
			is_record_select _
				=	False
	
			transform_record_update :: (Optional Ident) ParsedExpr ![[NestedUpdate]] !Int State -> (ParsedExpr, State)
			transform_record_update record_type expr groupedUpdates level pState
				=	(updateExpr, pState2)
				where
					/* final_record_type on a cycle */
					(assignments, (optionalIdent, final_record_type,pState2))
						= mapSt (transform_update level) groupedUpdates (No, record_type,pState)
					updateExpr
						= build_update final_record_type optionalIdent expr assignments
					// transform one group of nested updates with the same first field
					//  for example: f.g1 = e1, f.g2 = e2 -> f = {id.f & g1 = e1, g2 = e2},
					//  (id is ident to shared expression that's being updated)
	
					transform_update :: !Int [NestedUpdate] (Optional Ident,Optional Ident,State) -> (FieldAssignment, !(!Optional Ident,!Optional Ident,State))
					transform_update _ [{nu_selectors=[PS_Record fieldIdent field_record_type], nu_update_expr}] (shareIdent,record_type,pState)
						# (record_type,pState) = check_field_and_record_types field_record_type record_type pState
						= ({bind_dst = fieldIdent, bind_src = nu_update_expr},(shareIdent,record_type,pState))
					transform_update level updates=:[{nu_selectors=[PS_Record fieldIdent field_record_type : _]} : _] (optionalIdent,record_type,pState)
						# (record_type,pState) = check_field_and_record_types field_record_type record_type pState
						# (shareIdent, pState)
							=	make_ident optionalIdent level pState
						  select
						  	=	PE_Selection ParsedNormalSelector (PE_Ident shareIdent) [PS_Record fieldIdent final_record_type]
						  (update_expr, pState)
						  	=	transform_record_or_array_update No select (map sub_update updates) (level+1) pState
						=	({bind_dst = fieldIdent, bind_src = update_expr}, (Yes shareIdent,record_type,pState))
						where
							make_ident :: (Optional Ident) !Int State -> (Ident, State)
							make_ident (Yes ident) _ pState
								=	(ident, pState)
							make_ident No level pState
								=	internalIdent ("s" +++ toString level +++ ";") pState
	
							sub_update :: NestedUpdate -> NestedUpdate
							sub_update update=:{nu_selectors}
								=	{update & nu_selectors = tl nu_selectors}
					transform_update _ _ (_, record_type,pState)
						# pState
							=	parseError "record or array" No "field assignments mixed with array assignments not" pState
						=	({bind_dst = errorIdent, bind_src = PE_Empty}, (No,record_type,pState))
	
					build_update :: !(Optional Ident) !(Optional Ident) !ParsedExpr ![FieldAssignment] -> ParsedExpr
					build_update record_type No expr assignments
						=	PE_Record expr record_type assignments
					build_update record_type (Yes ident) expr assignments
						=	PE_Let False (LocalParsedDefs [buildNodeDef (PE_Ident ident) expr])
									(PE_Record (PE_Ident ident) record_type assignments)
					
					check_field_and_record_types :: (Optional Ident) (Optional Ident) State -> (!Optional Ident,!State);
					check_field_and_record_types No record_type pState
						= (record_type,pState);
					check_field_and_record_types field_record_type=:(Yes _) No pState
						= (field_record_type,pState);
					check_field_and_record_types (Yes field_record_type_name) record_type=:(Yes record_type_name) pState
						| field_record_type_name==record_type_name
							= (record_type,pState)
							# error_message = "record type in update: "+++field_record_type_name.id_name+++" where "+++record_type_name.id_name+++" was"
							= (record_type,parseError "record or array" No error_message pState)
	
			transform_array_update :: ParsedExpr [NestedUpdate] !Int State -> (ParsedExpr, State)
			transform_array_update expr updates level pState
				// transform {<e> & [i].<.&> = e1, ... } to  {{<e> & [i1].<.&> = e1} & ...}
				=	foldSt (transform_update level) updates (expr, pState)
				where
					transform_update :: !Int NestedUpdate (ParsedExpr, State) -> (ParsedExpr, State)
					transform_update level {nu_selectors, nu_update_expr} (expr1, pState)
						=	build_update expr1 (split_selectors nu_selectors) nu_update_expr level pState
						where
							// split selectors into final record selectors and initial selectors
							//  (resulting selectors are reversed)
							//		for example: [i1].[i2].f.[i3].g.h -> (h.g, [i3].f.[i2].[i1])
							split_selectors selectors
								=	span is_record_select (reverse selectors)
	
							build_update :: ParsedExpr ([ParsedSelection], [ParsedSelection]) ParsedExpr !Int State -> (ParsedExpr, State)
							build_update expr ([], initial_selectors) update_expr _ pState
								=	(PE_Update expr (reverse initial_selectors) update_expr, pState)
							// transform {<e> & <.&>.[i].f.g. = e1} to
							//     let
							//		index_id = i
							//		(element_id, array_id) = <e>!<.&>.[index_id]
							//	   in {array_id & [index_id] = {element_id & f.g = e1}}
							build_update expr (record_selectors, [PS_Array index : initial_selectors]) update_expr level pState
								# (index_id, pState)
									=	internalIdent ("i" +++ toString level +++ ";") pState
								# (element_id, pState)
									=	internalIdent ("e" +++ toString level +++ ";") pState
								# (array_id, pState)
									=	internalIdent ("a" +++ toString level +++ ";") pState
								  index_def
								  	=	buildNodeDef (PE_Ident index_id) index
								  select_def
								  	=	buildNodeDef
								  			(PE_Tuple [PE_Ident element_id, PE_Ident array_id])
								  			(PE_Selection (ParsedUniqueSelector True) expr (reverse [PS_Array (PE_Ident index_id) : initial_selectors]))
								  (updated_element, pState)
									= transform_record_update No
										(PE_Ident element_id)
										[[{nu_selectors=(reverse record_selectors), nu_update_expr=update_expr}]] (level+1) pState
								=	(PE_Let False
										(LocalParsedDefs [index_def, select_def])
										(PE_Update (PE_Ident array_id) (reverse [PS_Array (PE_Ident index_id) : initial_selectors]) updated_element), pState)

listExpr :: Try_0_1 ParsedExpr e
listExpr =
	keyword '['			&>
	strict	&>
	(	keyword ']' .: PE_Ident predefined_idents.[PD_NilSymbol]
	|>	keyword ",]"	&>
		groupWith graphExpr succeed &>
		apply_0 (PE_Ident predefined_idents.[PD_NilSymbol]) &>
		apply_2 (\l exp= foldr genNodes exp l)
	|>	listWith`` ',' graphExpr succeed &>
		(	keyword ":" &>
			graphExpr	&>
			keyword` ']' &>
			apply_2 (\l exp= foldr genNodes exp l)
		|>	keyword ".." &>
			(	keyword ']' &>
				apply_1 ff`
			|>	graphExpr &>
				keyword` ']' &>
				apply_2 ff
			)
		|>	keyword "\\\\" &>
			listWith` ',' qualifier ']' &>
			apply_2 c_compr
		|>	apply_0 (PE_Ident predefined_idents.[PD_NilSymbol]) &>
			keyword` ']' &>
			apply_2 (\l exp= foldr genNodes exp l)
		)
	)
where
	c_compr [exp] qualifiers
		# (cons_index,nil_index) = cons_and_nil_symbol_index HeadLazy
		= PE_ListCompr cons_index nil_index exp qualifiers
	
	genNodes hd exp= PE_List [makeConsIdentExpression HeadLazy True,hd,exp]
	ff [e] exp = PE_Sequ (SQ_FromTo e exp)
	ff [e2,e1] exp = PE_Sequ (SQ_FromThenTo e1 e2 exp)
	ff` [e] = PE_Sequ (SQ_From e)
	ff` [e2,e1] = PE_Sequ (SQ_FromThen e1 e2)

qualifier :: Simple_0_1 Qualifier e
qualifier =
	getLineAndColumn &>
	getFileName &>
	listWith`` "&" generator succeed &>
	(	keyword "|" &>
		graphExpr &>
		apply_1 Yes
	|>	apply_0 No
	) &>
	apply_4 \pos fileName generators filter=
		{qual_generators= generators, qual_filter= filter, qual_position= pos, qual_filename= fileName}
where
	generator =
		graphPattern &>
		(	keyword "<-:" .: IsArrayGenerator
		|>	keyword` "<-" .: IsListGenerator
		) &>
		getLineAndColumn &>
		graphExpr &>
		apply_4 \patt kind pos expr= {gen_kind= kind, gen_expr= expr, gen_pattern= patt, gen_position =pos}

/////////
/////////

parseError s1 x s2 state = expected (s1+++s2) id state
//parseErrorSimple s1 s2 state = expected (s1+++s2) id state
//"simple type" No ("More type attributes, "+toString attr1+" and "+toString attr2+", than") pState)
//:: *State :== *State
instance getPosition State
where
	getPosition state= getFilePosition (\state pos=(pos,state)) state

/////////////
//almost original
/////////////

module_id mod_name continue pState
//	| not iclmodule
//		= (defs,pState);

	// It is essential that the type name denoted by ident is an unique type name within the application. Otherwise
	// the static linker will choose one implementation (because the type names are equal) and map the other to the
	// chosen implementation.
	// The zero arity of the _Module constructor makes the code generator, pre-allocate _Module in .data section of
	// the final executable. The module name needed by the dynamic run-time system can then be determined by looking
	// at the descriptor. If however all implementations were mapped to a single one, the dynamic rts could not use
	// the module name anymore because they are all the same.
	# (ident,   pState)	= stringToIdent ("_" +++ mod_name +++ "_Module") IC_Type pState
	# td				= MakeTypeDef ident [] (ConsList []) TA_None [] NoPos
		
	# (pc_cons_name, pState) = stringToIdent "__Module" IC_Expression pState
	# cons
		= { pc_cons_name		= pc_cons_name
		,	pc_arg_types		= []
		,	pc_args_strictness	= NotStrict
		,	pc_cons_arity		= 0
		,	pc_cons_prio		= NoPrio
		,	pc_exi_vars			= []
		,	pc_cons_pos			= NoPos
		}
	# td = PD_Type { td & td_rhs = ConsList [cons] }
	= continue pState td


//////////////
//////////////

toIdent`` ident_class continue state name
	= into_hashTable ident_class (\state= continue state name) state name

internIdent s continue state
	# (ident,state) = internalIdent s state
	= continue state ident

internalIdent :: !String !*State -> (!Ident, !*State)
internalIdent prefix pState
	# ({fp_line,fp_col},pState)	= getPosition pState
	  case_string	= prefix +++ ";" +++ toString fp_line +++ ";" +++ toString fp_col
	= stringToIdent case_string IC_Expression pState

erroneousIdent = { id_name = "", id_info = nilPtr }

:: SAType = {s_annotation::!Annotation,s_type::!AType}

stringToIdent s i p :== into_hashTable i (\state i=(i,state)) p s

atypes_from_satypes :: ![SAType] -> [AType]
atypes_from_satypes []
	= []
atypes_from_satypes [{s_type}:types]
	= [s_type:atypes_from_satypes types]

strictness_from_satypes types
	= add_strictness_for_arguments types 0 0 NotStrict
where 
	add_strictness_for_arguments :: ![SAType] !Int !Int !StrictnessList -> StrictnessList
	add_strictness_for_arguments [] strictness_index strictness strictness_list
		| strictness==0
			= strictness_list
			= append_strictness strictness strictness_list
	add_strictness_for_arguments [{s_annotation=AN_Strict}:types] strictness_index strictness strictness_list
		# (strictness_index,strictness,strictness_list) = add_next_strict strictness_index strictness strictness_list
		= add_strictness_for_arguments types strictness_index strictness strictness_list
	add_strictness_for_arguments [{s_annotation=AN_None}:types] strictness_index strictness strictness_list
		# (strictness_index,strictness,strictness_list) = add_next_not_strict strictness_index strictness strictness_list
		= add_strictness_for_arguments types strictness_index strictness strictness_list

makeStringType
	#! string_ident = predefined_idents.[PD_StringType]
	=: TA (MakeNewTypeSymbIdent string_ident 0) []

makeListTypeSymbol :: Int Int -> TypeSymbIdent
makeListTypeSymbol head_strictness arity
	# pre_def_list_index=if (head_strictness==HeadLazy)
							PD_ListType
						(if (head_strictness==HeadStrict)
							PD_StrictListType
							PD_UnboxedListType)
	#! list_ident = predefined_idents.[pre_def_list_index]
	= MakeNewTypeSymbIdent list_ident arity

makeTailStrictListTypeSymbol :: Int Int -> TypeSymbIdent
makeTailStrictListTypeSymbol head_strictness arity
	# pre_def_list_index=if (head_strictness==HeadLazy)
							PD_TailStrictListType
						(if (head_strictness==HeadStrict)
							PD_StrictTailStrictListType
							PD_UnboxedTailStrictListType)
	#! list_ident = predefined_idents.[pre_def_list_index]
	= MakeNewTypeSymbIdent list_ident arity

makeLazyArraySymbol arity
	#! lazy_array_ident = predefined_idents.[PD_LazyArrayType]
	= MakeNewTypeSymbIdent lazy_array_ident arity

makeStrictArraySymbol arity
	#! strict_array_ident = predefined_idents.[PD_StrictArrayType]
	= MakeNewTypeSymbIdent strict_array_ident arity

makeUnboxedArraySymbol arity
	#! unboxed_array_ident = predefined_idents.[PD_UnboxedArrayType]
	= MakeNewTypeSymbIdent unboxed_array_ident arity

makeTupleTypeSymbol form_arity act_arity
	#! tuple_ident = predefined_idents.[GetTupleTypeIndex form_arity]
	= MakeNewTypeSymbIdent tuple_ident act_arity
	
HeadLazy:==0
HeadStrict:==1
HeadUnboxed:==2
HeadOverloaded:==3;
HeadUnboxedAndTailStrict:==4;

makeAttributeVar name :== { av_name = name, av_info_ptr = nilPtr }

makeNilExpression :: Int Bool -> ParsedExpr
makeNilExpression head_strictness is_pattern
	# pre_def_nil_index= if (head_strictness==HeadLazy)
							PD_NilSymbol
						(if (head_strictness==HeadStrict)
							PD_StrictNilSymbol
						(if (head_strictness==HeadOverloaded)
							(if is_pattern PD_OverloadedNilSymbol PD_nil)
							(if is_pattern PD_UnboxedNilSymbol PD_nil_u)))
	#! nil_ident = predefined_idents.[pre_def_nil_index]
	= PE_Ident nil_ident

makeTailStrictNilExpression :: Int Bool -> ParsedExpr
makeTailStrictNilExpression head_strictness is_pattern
	# pre_def_nil_index= if (head_strictness==HeadLazy)
							PD_TailStrictNilSymbol
						(if (head_strictness==HeadStrict)
							PD_StrictTailStrictNilSymbol
							(if is_pattern PD_UnboxedTailStrictNilSymbol PD_nil_uts))
	#! nil_ident = predefined_idents.[pre_def_nil_index]
	= PE_Ident nil_ident

makeConsIdentExpression :: Int Bool -> ParsedExpr
makeConsIdentExpression head_strictness is_pattern
	# pre_def_cons_index=if (head_strictness==HeadLazy)
							PD_ConsSymbol
						(if (head_strictness==HeadStrict)
							PD_StrictConsSymbol
						(if (head_strictness==HeadOverloaded)
							(if is_pattern PD_OverloadedConsSymbol PD_cons)
							(if is_pattern PD_UnboxedConsSymbol PD_cons_u)))
	#! cons_ident = predefined_idents.[pre_def_cons_index]
	= PE_Ident cons_ident

cons_and_nil_symbol_index HeadLazy = (PD_ConsSymbol,PD_NilSymbol)
cons_and_nil_symbol_index HeadStrict = (PD_StrictConsSymbol,PD_StrictNilSymbol)
cons_and_nil_symbol_index HeadUnboxed = (PD_cons_u,PD_nil_u)
cons_and_nil_symbol_index HeadOverloaded = (PD_cons,PD_nil)

makeTailStrictConsIdentExpression :: Int Bool -> ParsedExpr
makeTailStrictConsIdentExpression head_strictness is_pattern
	# pre_def_cons_index=if (head_strictness==HeadLazy)
							PD_TailStrictConsSymbol
						(if (head_strictness==HeadStrict)
							PD_StrictTailStrictConsSymbol
							(if is_pattern PD_UnboxedTailStrictConsSymbol PD_cons_uts))
	#! cons_ident = predefined_idents.[pre_def_cons_index]
	= PE_Ident cons_ident

tail_strict_cons_and_nil_symbol_index HeadLazy = (PD_TailStrictConsSymbol,PD_TailStrictNilSymbol)
tail_strict_cons_and_nil_symbol_index HeadStrict = (PD_StrictTailStrictConsSymbol,PD_StrictTailStrictNilSymbol)
tail_strict_cons_and_nil_symbol_index HeadUnboxed = (PD_cons_uts,PD_nil_uts)

:: NestedUpdate =
	{	nu_selectors :: ![ParsedSelection]
	,	nu_update_expr :: !ParsedExpr
	}

errorIdent :: Ident
errorIdent
	=	{id_name = "<<error>>", id_info = nilPtr}

groupBy :: (a a -> Bool) [a] -> [[a]]
groupBy eq []
    =   []
groupBy eq [h : t]
    =   [[h : this] : groupBy eq other]
    where
        (this, other)
            =   span (eq h) t

buildNodeDef :: ParsedExpr ParsedExpr -> ParsedDefinition
buildNodeDef lhsExpr rhsExpr
	=	PD_NodeDef NoPos lhsExpr rhs
	where
		rhs	=
			{ rhs_alts
				= UnGuardedExpr
					{ ewl_nodes		= []
					, ewl_locals	= LocalParsedDefs []
					, ewl_expr		= rhsExpr
					, ewl_position	= NoPos
					}
			, rhs_locals
				= LocalParsedDefs []
			}

